@@ -59,7 +59,7 @@ class Sequence(Generic[T], Iterable[T]):
         >>> it(lst).filter_not(lambda x: x.startswith('a'))
         ['b1', 'b2']
         """
-        return Sequence(filter(lambda x: not predicate(x), self._iter))
+        return self.filter(lambda x: not predicate(x))
 
     def filter_not_none(self) -> Sequence[T]:
         """
@@ -452,12 +452,7 @@ class Sequence(Generic[T], Iterable[T]):
         if n == 0:
             return self
 
-        lst = []
-        for i, e in enumerate(self):
-            if i < n:
-                continue
-            lst.append(e)
-        return it(lst)
+        return DropSequence(self, n)
 
     # noinspection PyShadowingNames
     def drop_while(self, predicate: Callable[[T], bool]) -> Sequence[T]:
@@ -465,16 +460,45 @@ class Sequence(Generic[T], Iterable[T]):
          Returns a Sequence containing all elements except first elements that satisfy the given [predicate].
 
          Example 1:
-        >>> lst = [1, 2, 3, 4, 5]
+        >>> lst = [1, 2, 3, 4, 1]
         >>> it(lst).drop_while(lambda x: x < 3 )
-        [3, 4, 5]
+        [3, 4, 1]
         """
-        lst = []
-        for e in self:
-            if predicate(e):
-                continue
-            lst.append(e)
-        return it(lst)
+        return DropWhileSequence(self, predicate)
+    
+    def skip(self, n: int) -> Sequence[T]:
+        """
+         Returns a Sequence containing all elements except first [n] elements.
+
+         Example 1:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).skip(0)
+        ['a', 'b', 'c']
+
+         Example 2:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).skip(1)
+        ['b', 'c']
+
+
+         Example 2:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).skip(4)
+        []
+        """
+        return self.drop(n)
+    
+    def skip_while(self, predicate: Callable[[T], bool]) -> Sequence[T]:
+        """
+         Returns a Sequence containing all elements except first elements that satisfy the given [predicate].
+
+         Example 1:
+        >>> lst = [1, 2, 3, 4, 1]
+        >>> it(lst).skip_while(lambda x: x < 3 )
+        [3, 4, 1]
+        """
+        return self.drop_while(predicate)
+
 
     def take(self, n: int) -> Sequence[T]:
         """
@@ -496,12 +520,7 @@ class Sequence(Generic[T], Iterable[T]):
             raise ValueError(f'Requested element count {n} is less than zero.')
         if n == 0:
             return Sequence([])
-        lst = []
-        for i, e in enumerate(self, 1):
-            lst.append(e)
-            if i == n:
-                break
-        return it(lst)
+        return TakeSequence(self, n)
 
     # noinspection PyShadowingNames
     def take_while(self, predicate: Callable[[T], bool]) -> Sequence[T]:
@@ -514,14 +533,30 @@ class Sequence(Generic[T], Iterable[T]):
         ['a', 'b']
 
         """
+        return TakeWhileSequence(self, predicate)
+    
+    def take_last(self, n: int) -> Sequence[T]:
+        """
+         Returns an Sequence containing last [n] elements.
 
-        lst = []
-        for e in self:
-            if not predicate(e):
-                break
-            lst.append(e)
+         Example 1:
+        >>> a = ['a', 'b', 'c']
+        >>> it(a).take_last(0)
+        []
 
-        return it(lst)
+
+         Example 2:
+        >>> a = ['a', 'b', 'c']
+        >>> it(a).take_last(2)
+        ['b', 'c']
+
+        """
+        if n < 0:
+            raise ValueError(f'Requested element count {n} is less than zero.')
+        if n == 0:
+            return Sequence([])
+
+        return self.drop(len(self) - n)
 
     # noinspection PyShadowingNames
     def sorted(self) -> Sequence[T]:
@@ -907,6 +942,30 @@ class Sequence(Generic[T], Iterable[T]):
         """
         for i, e in enumerate(self):
             action(e, i)
+    
+    @overload
+    def zip(self, other: Sequence[T]) -> Sequence[Tuple[T, T]]:
+        """
+         Returns a new Sequence of tuples, where each tuple contains two elements.
+        
+         Example 1:
+        >>> lst1 = ['a', 'b', 'c']
+        >>> lst2 = [1, 2, 3]
+        >>> it(lst1).zip(lst2)
+        [('a', 1), ('b', 2), ('c', 3)]
+
+         Example 2:
+        >>> lst1 = ['a', 'b', 'c']
+        >>> lst2 = [1, 2, 3]
+        >>> it(lst1).zip(lst2, lambda x, y: x + '__' +str( y))
+        ['a__1', 'b__2', 'c__3']
+        """
+        ...
+    @overload
+    def zip(self, other: Iterable[R], transform: Callable[[T, R], V]) -> Sequence[V]:
+        ...
+    def zip(self, other: Iterable[R], transform: Callable[[T, R], V] = lambda a, b:(a, b)) -> Sequence[V]:
+        return MergingSequence(self, other, transform)
 
     def to_set(self) -> Set[T]:
         """
@@ -1015,6 +1074,110 @@ class FlatteningSequence(Sequence[R]):
             yield from self._transformer(i)
 
 
+class DropSequence(Sequence[T]):
+    def __init__(self, iterable: Iterable[T], n: int) -> None:
+        self._iter = iterable
+        self._n = n
+        
+    def __iter__(self) -> Iterator[T]:
+        for i, e in enumerate(self._iter):
+            if i < self._n:
+                continue
+            yield e
+
+
+class DropWhileSequence(Sequence[T]):
+    def __init__(self, iterable: Iterable[T], predicate: Callable[[T], bool]) -> None:
+        self._iter = iterable
+        self._predicate = predicate
+    
+    def __iter__(self) -> Iterator[T]:
+        drop_state = True
+        for e in self._iter:
+            if drop_state and self._predicate(e):
+                continue
+            else:
+                drop_state = False
+            yield e
+
+
+class TakeSequence(Sequence[T]):
+    def __init__(self, iterable: Iterable[T], n: int) -> None:
+        self._iter = iterable
+        self._n = n
+    
+    def __iter__(self) -> Iterator[T]:
+        for i, e in enumerate(self._iter):
+            if i >= self._n:
+                break
+            yield e
+
+
+class TakeWhileSequence(Sequence[T]):
+    def __init__(self, iterable: Iterable[T], predicate: Callable[[T], bool]) -> None:
+        self._iter = iterable
+        self._predicate = predicate
+    
+    def __iter__(self) -> Iterator[T]:
+        take_state = True
+        for e in self._iter:
+            if take_state and self._predicate(e):
+                yield e
+            else:
+                break
+
+
+class MergingSequence(Sequence[V]):
+    def __init__(self, 
+    iterable: Iterable[T], 
+    other: Iterable[R],
+    transformer: Callable[[T, R], V] = lambda a,b:(a,b)) -> None:
+        self._iter = iterable
+        self._other = other
+        self._transformer = transformer
+    
+    def __iter__(self) -> Iterator[V]:
+        iter1 = iter(self._iter)
+        iter2 = iter(self._other)
+        while True:
+            try:
+                yield self._transformer(next(iter1), next(iter2))
+            except StopIteration:
+                break
+
+
+def sequence(iterable: Iterable[T]) -> Sequence[T]:
+    """
+     Returns a Sequence with elements of the given Iterable.
+
+     Example 1:
+    >>> seq(['a', 'b', 'c'])
+    ['a', 'b', 'c']
+    """
+    return Sequence(iterable)
+
+
+def seq(iterable: Iterable[T]) -> Sequence[T]:
+    """
+     Returns a Sequence with elements of the given Iterable.
+
+     Example 1:
+    >>> it(['a', 'b', 'c'])
+    ['a', 'b', 'c']
+    """
+    return sequence(iterable)
+
+
+def iterate(iterable: Iterable[T]) -> Sequence[T]:
+    """
+     Returns a Sequence with elements of the given Iterable.
+
+     Example 1:
+    >>> it(['a', 'b', 'c'])
+    ['a', 'b', 'c']
+    """
+    return sequence(iterable)
+
 
 def it(iterable: Iterable[T]) -> Sequence[T]:
     """
@@ -1024,17 +1187,9 @@ def it(iterable: Iterable[T]) -> Sequence[T]:
     >>> it(['a', 'b', 'c'])
     ['a', 'b', 'c']
     """
-    return Sequence(iterable)
+    return sequence(iterable)
 
-def seq(iterable: Iterable[T]) -> Sequence[T]:
-    """
-     Returns a Sequence with elements of the given Iterable.
 
-     Example 1:
-    >>> seq(['a', 'b', 'c'])
-    ['a', 'b', 'c']
-    """
-    return it(iterable)
 
 if __name__ == "__main__":
     import doctest
